package com.jiuqi.grid;

import com.jiuqi.util.StreamException;
import com.jiuqi.util.Stream;
import java.util.ArrayList;
import java.util.List;
import java.io.Serializable;

public class ValueList implements Serializable
{
    private List list;
    private List extList;
    private static final int MaxWordSign = 65535;
    
    public ValueList() {
        this.list = new ArrayList();
        this.extList = new ArrayList();
    }
    
    public String get(final int index) {
        return this.list.get(index).value;
    }
    
    public int add(final CellValue cv) {
        this.list.add(cv);
        if (cv.col > 65535 || cv.row > 65535) {
            this.extList.add(cv);
        }
        return this.list.size() - 1;
    }
    
    public int col(final int index) {
        return this.list.get(index).col;
    }
    
    public int row(final int index) {
        return this.list.get(index).row;
    }
    
    public int count() {
        return this.list.size();
    }
    
    public void clear() {
        this.extList.clear();
        this.list.clear();
    }
    
    public CellValue find(final int col, final int row) {
        for (int i = 0; i < this.list.size(); ++i) {
            final CellValue cv = this.list.get(i);
            if (cv.col == col && cv.row == row) {
                return cv;
            }
        }
        return null;
    }
    
    public void loadFromStream(final Stream stream) throws StreamException {
        this.clear();
        int c = stream.readInt();
        int l = 0;
        for (int i = 0; i < c; ++i) {
            final CellValue cv = new CellValue();
            cv.col = stream.readWord();
            cv.row = stream.readWord();
            l = stream.readWord();
            if (l == 65535) {
                l = stream.readInt();
            }
            cv.value = stream.readString(l);
            this.list.add(cv);
        }
        if (stream.getPosition() + 4L <= stream.getSize()) {
            c = stream.readInt();
            for (int i = 0; i < c; ++i) {
                final CellValue cv = new CellValue();
                cv.col = stream.readInt();
                cv.row = stream.readInt();
                l = stream.readWord();
                if (l == 65535) {
                    l = stream.readInt();
                }
                cv.value = stream.readString(l);
                this.list.add(cv);
                this.extList.add(cv);
            }
        }
    }
    
    public void saveToStream(final Stream stream) throws StreamException {
        stream.writeInt(this.list.size() - this.extList.size());
        for (int i = 0; i < this.list.size(); ++i) {
            final CellValue cv = this.list.get(i);
            if (cv.col <= 65535) {
                if (cv.row <= 65535) {
                    stream.writeWord(cv.col);
                    stream.writeWord(cv.row);
                    if (cv.value != null && cv.value.length() > 0) {
                        final byte[] b = cv.value.getBytes();
                        if (b.length < 65535) {
                            stream.writeWord(b.length);
                        }
                        else {
                            stream.writeWord(65535);
                            stream.writeInt(b.length);
                        }
                        stream.writeBuffer(b, 0, b.length);
                    }
                    else {
                        stream.writeWord(0);
                    }
                }
            }
        }
        if (this.extList.size() != 0) {
            stream.writeInt(this.extList.size());
            for (int i = 0; i < this.extList.size(); ++i) {
                final CellValue cv = this.extList.get(i);
                stream.writeInt(cv.col);
                stream.writeInt(cv.row);
                if (cv.value != null && cv.value.length() > 0) {
                    final byte[] b = cv.value.getBytes();
                    if (b.length < 65535) {
                        stream.writeWord(b.length);
                    }
                    else {
                        stream.writeWord(65535);
                        stream.writeInt(b.length);
                    }
                    stream.writeBuffer(b, 0, b.length);
                }
                else {
                    stream.writeWord(0);
                }
            }
        }
    }
}
